﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using NFC_4Developers.Resources;

namespace NFC_4Developers
{
    using System.Runtime.InteropServices.WindowsRuntime;
    using System.Text;

    using Microsoft.Phone.Tasks;

    using NdefLibrary.Ndef;

    using Windows.Networking.Proximity;
    using Windows.Networking.Sockets;

    public partial class MainPage : PhoneApplicationPage
    {
        private ProximityDevice proximityDevice = ProximityDevice.GetDefault();

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            // Sample code to localize the ApplicationBar
            //BuildLocalizedApplicationBar();
        }

        #region Demo 1 - udostępnij zdjęcie
        
        private void Demo1_SharePhoto(object sender, RoutedEventArgs e)
        {
            CameraCaptureTask cameraCaptureTask = new CameraCaptureTask();
            cameraCaptureTask.Completed += (senderObject, photoResult) =>
                {
                    if (photoResult.TaskResult == TaskResult.OK)
                    {
                        ShareMediaTask shareMediaTask = new ShareMediaTask();
                        shareMediaTask.FilePath = photoResult.OriginalFileName;
                        shareMediaTask.Show();
                    }
                };

            cameraCaptureTask.Show();
        }

        #endregion

        #region Demo 2 - wykrywanie urządzenia NFC

        private void Demo2_NFCDeviceDetection(object sender, RoutedEventArgs e)
        {
            ProximityDevice proximityDevice = ProximityDevice.GetDefault();
            if (proximityDevice != null)
            {
                // Urządzenie ma moduł NFC
                proximityDevice.DeviceArrived += proximityDevice_DeviceArrived;
                proximityDevice.DeviceDeparted += proximityDevice_DeviceDeparted;
            }
            else
            {
                // Brak modułu NFC
            }
        }

        private void proximityDevice_DeviceArrived(ProximityDevice sender)
        {
            this.ShowMessage("Wykryto urządzania o ID " + sender.DeviceId);
        }

        private void proximityDevice_DeviceDeparted(ProximityDevice sender)
        {
            this.ShowMessage("Zgubiono urządzanie o ID " + sender.DeviceId);
        }

        #endregion

        #region Demo 3 - wykrywanie urządzenia NFC

        private void Demo3_NFCTagRead(object sender, RoutedEventArgs e)
        {
            ProximityDevice proximityDevice = ProximityDevice.GetDefault();
            if (proximityDevice != null)
            {
                long subscribedMessageId = proximityDevice.SubscribeForMessage("WriteableTag", messageReceivedHandler);
            }
        }

        private void messageReceivedHandler(ProximityDevice device, ProximityMessage message)
        {
            int writableSize = System.BitConverter.ToInt32(message.Data.ToArray(), 0);
            this.ShowMessage("MessageType: " + message.MessageType + "\n"
                            + "DataAsString:" + message.DataAsString + "\n"
                            + "SubscriptionId: " + message.SubscriptionId + "\n"
                            + "Size of tag: " + writableSize + "\n"
                );
        }

        #endregion

        #region Demo4 - Publikacja taga

        private long subscribedMessageId;

        private void Demo4_ToDeviceMessage(object sender, RoutedEventArgs e)
        {
            this.proximityDevice.PublishMessage("Windows.SampleMessage", "Hello World", MessageTransmittedHandler);
        }

        private void Demo4_ToDeviceURI(object sender, RoutedEventArgs e)
        {
            this.proximityDevice.PublishUriMessage(new System.Uri("http://www.onet.pl"), MessageTransmittedHandler);
        }

        private void Demo4_ToTagBinaryMessage(object sender, RoutedEventArgs e)
        {
            this.proximityDevice.PublishBinaryMessage("Windows:WriteTag.mySubType", Encoding.Unicode.GetBytes("Hello World").AsBuffer(), MessageTransmittedHandler);
        }

        private void Demo4_ToTagURI(object sender, RoutedEventArgs e)
        {
            Uri uri = new Uri("http://www.wp.pl");
            byte[] buffer = Encoding.Unicode.GetBytes(uri.ToString());
            proximityDevice.PublishBinaryMessage("WindowsUri:WriteTag", buffer.AsBuffer(), MessageTransmittedHandler);
        }

        private void Demo4_TagRead(object sender, RoutedEventArgs e)
        {
            this.subscribedMessageId = this.proximityDevice.SubscribeForMessage("Windows.mySubType", messageReceivedHandler2);
        }

        private void messageReceivedHandler2(ProximityDevice device, ProximityMessage message)
        {
            this.proximityDevice.StopSubscribingForMessage(this.subscribedMessageId);

            var buffer = message.Data.ToArray();

            this.ShowMessage("MessageType: " + message.MessageType + "\n"
                            + "Data:" + Encoding.Unicode.GetString(buffer, 0, buffer.Length) + "\n"
                            + "SubscriptionId: " + message.SubscriptionId + "\n"
                );
        }

        #endregion

        #region Demo 5 - Uruchomienie aplikacji

        private void Demo5_StartAppliationTagWrite(object sender, RoutedEventArgs e)
        {
            Uri uri = new Uri("ms-drive-to:?destination.latitude=52.199506&destination.longitude=21.041737&destination.name=4Developers");
            byte[] buffer = Encoding.Unicode.GetBytes(uri.ToString());
            proximityDevice.PublishBinaryMessage("WindowsUri:WriteTag", buffer.AsBuffer(), MessageTransmittedHandler);
        }

        #endregion

        #region Demo 6 - Własny schemat URI

        private void Demo6_SaveTagWithOwnUri(object sender, RoutedEventArgs e)
        {
            Uri uri = new Uri("developers:MainPage?Param=HelloWorld");
            byte[] buffer = Encoding.Unicode.GetBytes(uri.ToString());
            proximityDevice.PublishBinaryMessage("WindowsUri:WriteTag", buffer.AsBuffer(), MessageTransmittedHandler);
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            if (NavigationContext.QueryString.ContainsKey("Param"))
            {
                this.ShowMessage(NavigationContext.QueryString["Param"]);
            }

            base.OnNavigatedTo(e);
        }

        #endregion

        #region Demo 7 - Zapis wiadomości NDEF

        private void Demo7_SaveNDEF(object sender, RoutedEventArgs e)
        {
            NdefUriRecord uriRecord = new NdefUriRecord { Uri = "http://2013.4developers.org.pl" };
            NdefMessage message = new NdefMessage { uriRecord };
            ProximityDevice.GetDefault().PublishBinaryMessage("NDEF:WriteTag", message.ToByteArray().AsBuffer(), MessageTransmittedHandler);
        }

        private void Demo7_OpenNDEF(object sender, RoutedEventArgs e)
        {
            ProximityDevice.GetDefault().SubscribeForMessage("NDEF", MessageReceivedHandler);
        }

        private void MessageReceivedHandler(ProximityDevice sender, ProximityMessage message)
        {
            var rawMsg = message.Data.ToArray();
            var ndefMessage = NdefMessage.FromByteArray(rawMsg);

            foreach (NdefRecord record in ndefMessage)
            {
                if (NdefUriRecord.IsRecordType(record))
                {
                    var uriRecord = new NdefUriRecord(record);
                    ShowMessage("URI: " + uriRecord.Uri);
                }
            }

            sender.StopSubscribingForMessage(message.SubscriptionId);
        }

        #endregion

        #region Demo 8 - Automatyczne nawiązywanie połączenia

        private void Demo8_ConnectToOtherDevice(object sender, RoutedEventArgs e)
        {
            ProximityDevice device = ProximityDevice.GetDefault();
            if (device != null)
            {
                PeerFinder.TriggeredConnectionStateChanged += OnTriggeredConnectionStateChanged;
                PeerFinder.AllowBluetooth = true;
                PeerFinder.AllowWiFiDirect = true;
                PeerFinder.Start();
            }
        }

        StreamSocket streamSocket;

        private void OnTriggeredConnectionStateChanged(object sender, TriggeredConnectionStateChangedEventArgs args)
        {
            switch (args.State)
            {
                case TriggeredConnectState.Listening:
                    // Łączenie jako host
                    break;
                case TriggeredConnectState.PeerFound:
                    // Zakończono używania NFC - połączenie wykorzystuje Wi-Fi, Bluetooth.
                    // Można odsunąć urządzenia.
                    break;
                case TriggeredConnectState.Connecting:
                    // Łączenie...
                    break;
                case TriggeredConnectState.Completed:
                    // Połączenie ustanowiono
                    streamSocket = args.Socket;
                    break;
                case TriggeredConnectState.Canceled:
                    break;
                case TriggeredConnectState.Failed:
                    // Nastąpił błąd - trzeba jeszcze raz spróbować
                    break;
            }
        }

        #endregion

        #region Common

        private void ShowMessage(string text)
        {
            Dispatcher.BeginInvoke(() => this.tb_status.Text = "Status: " + text);
        }

        private void MessageTransmittedHandler(ProximityDevice sender, long messageId)
        {
            this.proximityDevice.StopPublishingMessage(messageId);
            this.ShowMessage("Wysłano / Zapisano!");
        }
        #endregion


    }
}